# Using RuleDispatch:

import datetime

import dispatch
from simplejson import JSONEncoder

try:
    from turbogears import config
except ImportError:
    class config(object):
        _default = object()
        def get(cls, key, default=_default):
            if default is cls._default:
                raise KeyError(key)
            return default
        get = classmethod(get)

try:
    import decimal
except ImportError:
    # Python 2.3
    decimal = None

def jsonify(obj):
    """
    Return an object that can be serialized with JSON, i.e., it
    is made up of only lists, dictionaries (with string keys),
    and strings, ints, and floats.
    """
    raise NotImplementedError
jsonify = dispatch.generic()(jsonify)

def jsonify_datetime(obj):
	return str(obj)
jsonify_datetime = jsonify.when(
        'isinstance(obj, datetime.datetime) or '
        'isinstance(obj, datetime.date)')(jsonify_datetime)

def jsonify_decimal(obj):
    return float(obj)
if decimal is not None:
    jsonify_decimal = jsonify.when('isinstance(obj, decimal.Decimal)')(
        jsonify_decimal)

def jsonify_explicit(obj):
    return obj.__json__()
jsonify_explicit = jsonify.when('hasattr(obj, "__json__")')(jsonify_explicit)

# SQLObject support
try:
    import sqlobject

    def jsonify_sqlobject(obj, descent_bases=[]):
        if not descent_bases:
            descent_bases.append(config.get('turbojson.descent_bases', True))
        descent_bases = descent_bases[0]
        result = {}
        result['id'] = obj.id
        keys = []
        sm = obj.__class__.sqlmeta
        try:
            while sm is not None:
                # we need to exclude the ID-keys, as for some reason
                # this won't work for subclassed items
                keys.extend([key for key in sm.columns.keys() if key[-2:] != 'ID'])
                if descent_bases:
                    sm = sm.__base__
                else:
                    sm = None
        except AttributeError:
            # this happens if we descent to <type object>
            pass
        for name in keys:
            result[name] = getattr(obj, name)
        return result
    jsonify_sqlobject = jsonify.when(
            'isinstance(obj, sqlobject.SQLObject)')(jsonify_sqlobject)

    def jsonify_select_results(obj):
        return list(obj)
    jsonify_select_results = jsonify.when(
            'isinstance(obj, sqlobject.SQLObject.SelectResultsClass)')(
                    jsonify_select_results)
except ImportError:
    pass

# SQLAlchemy support
try:
    import sqlalchemy

    def jsonify_saobject(obj):
       props = {}
       for key in obj.c.keys():
           props[key] = getattr(obj, key)
       return props
    jsonify_saobject = jsonify.when(
            "hasattr(obj, 'c') and isinstance(obj.c,sqlalchemy.util.OrderedProperties)")(jsonify_saobject)

except ImportError:
    pass


class GenericJSON(JSONEncoder):

    def default(self, obj):
        return jsonify(obj)

_instance = GenericJSON()

def encode_iter(obj):
    return _instance.iterencode(obj)

def encode(obj):
    return _instance.encode(obj)



from turbojson import jsonify

try:
    from sqlalchemy import *
    from sqlalchemy.orm import *
    gotsa = True
    
    metadata = MetaData('sqlite:///:memory:')
    test1 = Table('test1', metadata, 
            Column('id', Integer, primary_key=True),
            Column('val', String))

    test2 = Table('test2', metadata, 
            Column('id', Integer, primary_key=True),
            Column('test1id', Integer, ForeignKey('test1.id')),
            Column('val', String))
    metadata.create_all()
    
    class Test2(object):
        pass
    mapper(Test2, test2)

    class Test1(object):
        pass
    mapper(Test1, test1, properties={'test2s':relation(Test2)})

    test1.insert().execute({'id':1, 'val':'bob'})    
    test2.insert().execute({'id':1, 'test1id':1, 'val':'fred'})
    test2.insert().execute({'id':2, 'test1id':1, 'val':'alice'})
    
except ImportError:
    gotsa = False


def test_saobj():
    if not gotsa:
        return    
    s = create_session()
    t = s.query(Test1).get(1)
    encoded = jsonify.encode(t)
    assert encoded == '{"id": 1, "val": "bob"}'
    
def test_salist():
    if not gotsa:
        return    
    s = create_session()
    t = s.query(Test1).get(1)
    encoded = jsonify.encode(t.test2s)
    print encoded
    assert encoded == '[{"test1id": 1, "id": 1, "val": "fred"}, {"test1id": 1, "id": 2, "val": "alice"}]'
    
    


"Template support for Json"

from turbojson import jsonify

class JsonSupport(object):
    
    def __init__(self, extra_vars_func=None, options=None):
        pass
        
    def render(self, info, format=None, fragment=False, template=None):
        """Renders data in the desired format.
    
        @param info: the data itself
        @type info: dict
        @param format: not used
        @type format: string
        @param fragment: not used
        @type fragment: bool
        @param template: not used
        @type template: string
        """
        [info.pop(item) for item in info.copy() if (item.startswith("tg_") and item != "tg_flash")]

        return jsonify.encode(info)

    def get_content_type(self, user_agent):
        if "Opera" in user_agent.browser: 
            return "text/plain"
        else:
            return "text/javascript"





from setuptools import setup, find_packages

setup(
    name="TurboJson",
    version="1.1.2",
    description="Python template plugin that supports JSON",
    author="Elvelind Grandin",
    author_email="elvelind+turbogears@gmail.com",
    url="http://www.turbogears.org/docs/plugins/template.html",
    download_url="http://www.turbogears.org/download/",
    license="MIT",
    keywords=["python.templating.engines", "turbogears"],
    install_requires=["RuleDispatch"],
    zip_safe=False,
    packages=find_packages(),
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Framework :: TurboGears',
        'Environment :: Web Environment :: Buffet',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    entry_points="""
    [python.templating.engines]
    json = turbojson.jsonsupport:JsonSupport
    """,
    test_suite = 'nose.collector',
    )




from turbojson import jsonify


class Foo(object):
    def __init__(self, bar):
        self.bar = bar

def jsonify_foo(obj):
    return "foo-%s" % obj.bar
jsonify_foo = jsonify.jsonify.when("isinstance(obj, Foo)")(jsonify_foo)

def test_dictionary():
    d = {'a':1, 'b':2}
    encoded = jsonify.encode(d)
    print encoded
    assert encoded == '{"a": 1, "b": 2}'

def test_specificjson():
    a = Foo("baz")
    encoded = jsonify.encode(a)
    print encoded
    assert encoded == '"foo-baz"'

def test_specific_in_dict():
    a = Foo("baz")
    d = {"a":a}
    encoded = jsonify.encode(d)
    print encoded
    assert encoded == '{"a": "foo-baz"}'
    

def test_so():
    try:
        import sqlobject as so
        import pysqlite2
        from sqlobject.inheritance import InheritableSQLObject
    except ImportError:
        return
    so.sqlhub.processConnection = so.connectionForURI('sqlite:/:memory:')
    class Person(so.SQLObject):
        fname = so.StringCol()
        mi = so.StringCol(length=1, default=None)
        lname = so.StringCol()
    Person.createTable()
    p = Person(fname="Peter", mi="P", lname="Pasulke")
    pj = jsonify.jsonify(p)
    assert pj == {'fname' : "Peter", 'mi' : "P", 'lname' : "Pasulke", 'id' : 1}
    class A(InheritableSQLObject):
        foo = so.StringCol()

    A.createTable()
    class B(A):
        bar = so.StringCol()

    B.createTable()

    
    b = B(foo="foo", bar="bar")
    bj = jsonify.jsonify(b)
    print bj
    assert bj == {'foo' : "foo", 'bar' : "bar", 'id' : b.id, 'childName' : None}


